==> delegate8.11.0.tar.gz: delegate8.11.0/resolvy/dnsnode.c <==
/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI

Permission to use, copy, modify, and distribute this material for any
purpose and without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies, and
that the name of ETL not be used in advertising or publicity pertaining
to this material without the specific, prior written permission of an
authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	dnode.c
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
	convert domain name to (local) uniq identifier.
History:
	950820	created
//////////////////////////////////////////////////////////////////////#*/

#include <stdio.h>
#include <stdlib.h>
#include "ystring.h"
#include "dns.h" /* REVERSE_DOM */

typedef struct attr {
    struct attr	*a_peer;
	int	 a_flag;
	int	 a_ttl;
	int	 a_leng;
	char	 a_data[1];
} Attr;

typedef struct node {
	int	 n_nid;
	int	 n_flag;
  const	char	*n_label;
	int	 n_phase;
	Attr	*n_attr;
    struct node *n_parent;
    struct node *n_child;
    struct node *n_peer;
} Node;
#define NF_LEAF		1

static Node NoNode     = {0,NF_LEAF,"?"};
static Node TopNodeBuf = {1,NF_LEAF,""};
static Node *TopNode = &TopNodeBuf;
static Node *Nodes0[] = {&NoNode, &TopNodeBuf};
static Node **Nodes = Nodes0;
static int NodesA;
static int NodesX = 2;
static int NID = 1;

#define getnode(nid)	Nodes[nid]

static Node *newnode()
{	Node *Np;
	int ox,nsize;
	Node **nnodes;

	++NID;
	if( NodesX <= NID ){
		NodesX = ++NodesA * 32;
		nsize = NodesX * sizeof(Node*);
		if( Nodes == Nodes0 ){
			Nodes = (Node**)malloc(nsize);
			Nodes[0] = Nodes0[0];
			Nodes[1] = Nodes0[1];
		}else	Nodes = (Node**)realloc(Nodes,nsize);
		debug(DBG_ANY,"newnode() id=%d %d %X\n",NID,NodesX,Nodes);
	}

	Np = NewStruct(Node);
	Nodes[NID] = Np;
	Np->n_nid = NID;
	return Np;
}

static Node *nsearch(PCStr(name),int create)
{	const char *np;
	char oct;
	const char *labelv[32]; /**/
	CStr(labelb,512);
	refQStr(lp,labelb); /**/
	const char *xp = &labelb[sizeof(labelb)-1];
	const char *label;
	int lc,lx;
	Node *Np,*Lp,*Tp;

	np = name;
	for(lc = 0; *np; lc++){
		if( xp <= lp )
			break;
		if( elnumof(labelv) <= lc ){
			debug(DBG_FORCE,"FATAL: nsearch(%s) too deep\n",name);
			break;
		}
		labelv[lc] = (char*)lp;
		while( oct = *np ){
			np++;
			if( oct == '.' )
				break;
			setVStrPtrInc(lp,oct);
		}
		setVStrPtrInc(lp,0);
	}
	Np = Lp = TopNode;

	if( lc == 1 && labelv[0][0] == 0 ){
		/* ROOT "." */
	}else
	for(lx = lc - 1; 0 <= lx; lx--){
		label = labelv[lx];
		for( Lp = Np->n_child; Lp; Lp = Lp->n_peer ){
			if( strcasecmp(Lp->n_label,label) == 0 )
				break;
		}
		if( Lp == NULL ){
			if( !create )
				return &NoNode;

			Lp = newnode();
			Lp->n_label = stralloc(label);
			Lp->n_parent = Np;
			Tp = Np->n_child;
			Np->n_child = Lp;
			Lp->n_peer = Tp;
		}
		Np = Lp;
	}
	if( create )
		Np->n_flag |= NF_LEAF;
	return Np;
}
static int getnodename(Node *Np,PVStr(name))
{	refQStr(np,name); /**/
	int lc;

	for( lc = 0; Np; lc++ ){
		strcpy(np,Np->n_label);
		np += strlen(np);
		Np = Np->n_parent;
		if( Np == TopNode )
			break;
		setVStrPtrInc(np,'.');
	}
	setVStrEnd(np,0);
	return lc;
}

void gethostdomname(PVStr(name),PCStr(addr),int len,int type)
{	const unsigned char *ap;

	ap = (unsigned char*)addr;
	sprintf(name,"%d.%d.%d.%d.%s.",ap[3],ap[2],ap[1],ap[0],REVERSE_DOM);
}

void addr2dom(PCStr(addr),PVStr(inaddr),int isize)
{	int a1,a2,a3,a4;

	sscanf(addr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
	sprintf(inaddr,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
}

int DNS_putbyaddr(PCStr(addr))
{	CStr(name,256);
	int a1,a2,a3,a4;

	sscanf(addr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
	sprintf(name,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
	return DNS_putbyname(name);
}
int DNS_getbyaddr(PCStr(addr))
{	CStr(name,256);

	sprintf(name,"%s.%s",addr,REVERSE_DOM);
	return DNS_getbyname(name);
}

int DNS_putbyname(PCStr(name))
{	Node *Np;

	Np = nsearch(name,1);
	return Np->n_nid;
}
int DNS_getbyname(PCStr(name))
{	Node *Np;

	Np = nsearch(name,0);
	return Np->n_nid;
}
int DNS_parent(int nid)
{	Node *Np,*Pp;

	Np = getnode(nid);
	if( Pp = Np->n_parent )
		return Pp->n_nid;
	else	return 0;
}
int DNS_nodename(int nid,PVStr(name))
{
	return getnodename(getnode(nid),AVStr(name));
}
void DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng)
{	Node *Np;
	Attr *Sp,*Ap;

	Np = getnode(nid);

	for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
		if( flag == Ap->a_flag && 
		    leng == Ap->a_leng && bcmp(Ap->a_data,data,leng) == 0 )
			return;
	}

	Ap = (Attr*)malloc(sizeof(Attr)+leng);
	Ap->a_flag = flag;
	Ap->a_leng = leng;
	Ap->a_ttl  = ttl;
	bcopy(data,Ap->a_data,leng);

	Sp = Np->n_attr;
	Np->n_attr = Ap;
	Ap->a_peer = Sp;
}
int DNS_getattr(int nid,int flag,int ttl,int ac,char *av[])
{	Node *Np;
	Attr *Ap;
	int ai;

	Np = getnode(nid);
	ai = 0;
	for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
		if( flag & Ap->a_flag ){
			av[ai++] = Ap->a_data;
			if( ac <= ai )
				break;
		}
	}
	return ai;
}

int DNS_nodephase(int nid,int phase)
{	Node *Np;
	int ophase;

	Np = getnode(nid);
	ophase = Np->n_phase;
	Np->n_phase = phase;
	return ophase;
}

void DNS_nodedump(int nid)
{	Node *Np;
	CStr(name,512);
	Attr *Ap;
	const unsigned char *dp;

	Np = getnode(nid);
	if( Np->n_attr != NULL ){
		getnodename(Np,AVStr(name));
		fprintf(stderr,"%6d: %s\n",Np->n_nid,name);

		for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
			dp = (unsigned char*)Ap->a_data;
			fprintf(stderr,"%6s+ ATTR-%d=%d %d %d %d %d %d\n",
				"",Ap->a_flag,dp[0],dp[1],dp[2],dp[3],dp[4],dp[5]);
		}
	}
}
void DNS_dump()
{	int nid;
	Node *Np;

	for( nid = 1; nid <= NID; nid++ ){
		Np = getnode(nid);
		if( Np->n_flag & NF_LEAF )
			DNS_nodedump(Np->n_nid);
	}
}

==> delegate8.11.4.tar.gz: delegate8.11.4/resolvy/dnsnode.c <==
/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI

Permission to use, copy, modify, and distribute this material for any
purpose and without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies, and
that the name of ETL not be used in advertising or publicity pertaining
to this material without the specific, prior written permission of an
authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	dnode.c
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
	convert domain name to (local) uniq identifier.
History:
	950820	created
//////////////////////////////////////////////////////////////////////#*/

#include <stdio.h>
#include <stdlib.h>
#include "ystring.h"
#include "dns.h" /* REVERSE_DOM */
#define MAXLEV 36

typedef struct attr {
    struct attr	*a_peer;
	int	 a_flag;
	int	 a_ttl;
	int	 a_leng;
	char	 a_data[1];
} Attr;

typedef struct node {
	int	 n_nid;
	int	 n_flag;
  const	char	*n_label;
	int	 n_phase;
	Attr	*n_attr;
    struct node *n_parent;
    struct node *n_child;
    struct node *n_peer;
} Node;
#define NF_LEAF		1

static Node NoNode     = {0,NF_LEAF,"?"};
static Node TopNodeBuf = {1,NF_LEAF,""};
static Node *TopNode = &TopNodeBuf;
static Node *Nodes0[] = {&NoNode, &TopNodeBuf};
static Node **Nodes = Nodes0;
static int NodesA;
static int NodesX = 2;
static int NID = 1;

#define getnode(nid)	Nodes[nid]

static Node *newnode()
{	Node *Np;
	int ox,nsize;
	Node **nnodes;

	++NID;
	if( NodesX <= NID ){
		NodesX = ++NodesA * MAXLEV;
		nsize = NodesX * sizeof(Node*);
		if( Nodes == Nodes0 ){
			Nodes = (Node**)malloc(nsize);
			Nodes[0] = Nodes0[0];
			Nodes[1] = Nodes0[1];
		}else	Nodes = (Node**)realloc(Nodes,nsize);
		debug(DBG_ANY,"newnode() id=%d %d %X\n",NID,NodesX,Nodes);
	}

	Np = NewStruct(Node);
	Nodes[NID] = Np;
	Np->n_nid = NID;
	return Np;
}

static Node *nsearch(PCStr(name),int create)
{	const char *np;
	char oct;
	const char *labelv[MAXLEV]; /**/
	CStr(labelb,512);
	refQStr(lp,labelb); /**/
	const char *xp = &labelb[sizeof(labelb)-1];
	const char *label;
	int lc,lx;
	Node *Np,*Lp,*Tp;

	np = name;
	for(lc = 0; *np; lc++){
		if( xp <= lp )
			break;
		if( elnumof(labelv) <= lc ){
			debug(DBG_FORCE,"FATAL: nsearch(%s) too deep\n",name);
			break;
		}
		labelv[lc] = (char*)lp;
		while( oct = *np ){
			np++;
			if( oct == '.' )
				break;
			setVStrPtrInc(lp,oct);
		}
		setVStrPtrInc(lp,0);
	}
	Np = Lp = TopNode;

	if( lc == 1 && labelv[0][0] == 0 ){
		/* ROOT "." */
	}else
	for(lx = lc - 1; 0 <= lx; lx--){
		label = labelv[lx];
		for( Lp = Np->n_child; Lp; Lp = Lp->n_peer ){
			if( strcasecmp(Lp->n_label,label) == 0 )
				break;
		}
		if( Lp == NULL ){
			if( !create )
				return &NoNode;

			Lp = newnode();
			Lp->n_label = stralloc(label);
			Lp->n_parent = Np;
			Tp = Np->n_child;
			Np->n_child = Lp;
			Lp->n_peer = Tp;
		}
		Np = Lp;
	}
	if( create )
		Np->n_flag |= NF_LEAF;
	return Np;
}
static int getnodename(Node *Np,PVStr(name))
{	refQStr(np,name); /**/
	int lc;

	for( lc = 0; Np; lc++ ){
		strcpy(np,Np->n_label);
		np += strlen(np);
		Np = Np->n_parent;
		if( Np == TopNode )
			break;
		setVStrPtrInc(np,'.');
	}
	setVStrEnd(np,0);
	return lc;
}

void gethostdomname(PVStr(name),PCStr(addr),int len,int type)
{	const unsigned char *ap;

	ap = (unsigned char*)addr;
	sprintf(name,"%d.%d.%d.%d.%s.",ap[3],ap[2],ap[1],ap[0],REVERSE_DOM);
}

void addr2dom(PCStr(addr),PVStr(inaddr),int isize)
{	int a1,a2,a3,a4;

	sscanf(addr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
	sprintf(inaddr,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
}

int DNS_putbyaddr(PCStr(addr))
{	CStr(name,256);
	int a1,a2,a3,a4;

	sscanf(addr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
	sprintf(name,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
	return DNS_putbyname(name);
}
int DNS_getbyaddr(PCStr(addr))
{	CStr(name,256);

	sprintf(name,"%s.%s",addr,REVERSE_DOM);
	return DNS_getbyname(name);
}

int DNS_putbyname(PCStr(name))
{	Node *Np;

	Np = nsearch(name,1);
	return Np->n_nid;
}
int DNS_getbyname(PCStr(name))
{	Node *Np;

	Np = nsearch(name,0);
	return Np->n_nid;
}
int DNS_parent(int nid)
{	Node *Np,*Pp;

	Np = getnode(nid);
	if( Pp = Np->n_parent )
		return Pp->n_nid;
	else	return 0;
}
int DNS_nodename(int nid,PVStr(name))
{
	return getnodename(getnode(nid),AVStr(name));
}
void DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng)
{	Node *Np;
	Attr *Sp,*Ap;

	Np = getnode(nid);

	for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
		if( flag == Ap->a_flag && 
		    leng == Ap->a_leng && bcmp(Ap->a_data,data,leng) == 0 )
			return;
	}

	Ap = (Attr*)malloc(sizeof(Attr)+leng);
	Ap->a_flag = flag;
	Ap->a_leng = leng;
	Ap->a_ttl  = ttl;
	bcopy(data,Ap->a_data,leng);

	Sp = Np->n_attr;
	Np->n_attr = Ap;
	Ap->a_peer = Sp;
}
int DNS_getattr(int nid,int flag,int ttl,int ac,char *av[])
{	Node *Np;
	Attr *Ap;
	int ai;

	Np = getnode(nid);
	ai = 0;
	for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
		if( flag & Ap->a_flag ){
			av[ai++] = Ap->a_data;
			if( ac <= ai )
				break;
		}
	}
	return ai;
}

int DNS_nodephase(int nid,int phase)
{	Node *Np;
	int ophase;

	Np = getnode(nid);
	ophase = Np->n_phase;
	Np->n_phase = phase;
	return ophase;
}

void DNS_nodedump(int nid)
{	Node *Np;
	CStr(name,512);
	Attr *Ap;
	const unsigned char *dp;

	Np = getnode(nid);
	if( Np->n_attr != NULL ){
		getnodename(Np,AVStr(name));
		fprintf(stderr,"%6d: %s\n",Np->n_nid,name);

		for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
			dp = (unsigned char*)Ap->a_data;
			fprintf(stderr,"%6s+ ATTR-%d=%d %d %d %d %d %d\n",
				"",Ap->a_flag,dp[0],dp[1],dp[2],dp[3],dp[4],dp[5]);
		}
	}
}
void DNS_dump()
{	int nid;
	Node *Np;

	for( nid = 1; nid <= NID; nid++ ){
		Np = getnode(nid);
		if( Np->n_flag & NF_LEAF )
			DNS_nodedump(Np->n_nid);
	}
}
